# -*- coding: utf-8 -*-
"""Python package providing horrifying welcome messages for your application.
"""
__name__ = 'portent'
__author__ = 'Stephen Ogilvy'

import shutil
import sys

import portent.data as data
import portent.welcomes


def welcome(stream=sys.stdout,
            theme='basic', name=None, version=None,
            quote_theme=None, columns=None,
            ):

    # Only run the welcome screen once per session
    if data.HAS_PRINTED:
        return

    # Set the width
    if not columns:
        columns = shutil.get_terminal_size().columns

    # Get the relevant welcome function and call it
    f = None
    try:
        f = portent.welcomes.welcomes[theme]
    except KeyError as k:
        print(f'theme: {k} not recognised.')
    f(stream, name, version, quote_theme, columns)

    # Register that we have run a welcome message this session
    data.HAS_PRINTED = True

    # Reset the ANSI formatting of the stream.
    stream.write(data.RESET)




"""Welcome message functions for portent.
"""
import textwrap
from datetime import datetime

import portent.data as data
import portent.quotes as quotes


def welcome_message_basic(stream, name, version, quote_theme, columns):
    """Basic welcome screen."""
    if not quote_theme:
        quote_theme = 'satire'

    nc = int(columns / 2)

    q, a = quotes.get_quote_and_attribution_tuple(quote_theme)
    q = '"' + q + '"'
    a = f'- {a}'
    q_list = textwrap.wrap(q, nc - 5, break_long_words=False)

    stream.write(data.BOLD + '-'*nc+'\n')
    if name:
        stream.write(f': Welcome to {data.BLUE}{name}!\n')
    else:
        stream.write(': Welcome!\n')
    if version:
        stream.write(f'{data.WHITE}: Version: {data.CYAN}{version}\n')

    stream.write(f'{data.WHITE}: Inspirational quote of the day:\n')
    for l in q_list:
        stream.write(':   ' + data.MAGENTA + l + data.WHITE + '\n')

    attribution_length = nc
    if len(q) < nc:
        attribution_length = len(q) + 5

    stream.write(':' + data.PURPLE + a.rjust(attribution_length - 6) +
                 data.WHITE + '\n')
    stream.write('-'*nc+'\n')


def welcome_message_finance(stream, name, version, quote_theme, columns):
    """Finance themed welcome screen"""
    if not quote_theme:
        quote_theme = 'satire'

    stream.write(data.GREEN + data.BOLD)
    stream.write('$' * columns + '\n')
    stream.write(data.BOLD + data.GREEN)

    str1 = 'Welcome!'
    if name and version:
        str1 = f'Welcome to {name} v{version}'
    elif name:
        str1 = f'Welcome to {name}!'
    str2 = 'The great bulls of finance advise you:'
    str5 = datetime.utcnow().strftime('%D %T')
    q, a = quotes.get_quote_and_attribution_tuple(quote_theme)
    str3_base = '"' + q + '"'
    str3_list = textwrap.wrap(str3_base, columns - 10, break_long_words=False)
    str4 = f'- {a}'

    stream.write('$$$' + ''.center(columns - 6, ' ') + '$$$\n')
    stream.write('$$$' + data.RED + str1.center(columns - 6, ' ')
                 + data.GREEN + '$$$\n')
    stream.write('$$$' + data.RED + str5.center(columns - 6, ' ')
                 + data.GREEN + '$$$\n')
    stream.write('$$$' + data.RED + str2.center(columns - 6, ' ')
                 + data.GREEN + '$$$\n')
    stream.write('$$$' + ''.center(columns - 6, ' ') + '$$$\n')
    for str3 in str3_list:
        stream.write('$$$' + data.WHITE + str3.center(columns - 6, ' ')
                     + data.GREEN + '$$$\n')
    stream.write('$$$ ' + data.WHITE + str4.rjust(columns - 15, ' ')
                 + data.GREEN + '        $$$\n')
    stream.write('$$$' + ''.center(columns - 6, ' ') + '$$$\n')
    stream.write(data.GREEN)
    stream.write('$' * columns + '\n')
    stream.write(data.RESET)


welcomes = {
    'basic': welcome_message_basic,
    'finance': welcome_message_finance,
}



"""ANSI codes and static text for portent's welcome messages.
"""

# 8-bit foreground colour codes.
RED = '\033[38;5;196m'
YELLOW = '\033[38;5;226m'
WHITE = '\033[38;5;255m'
BLUE = '\033[38;5;33m'
CYAN = '\033[38;5;51m'
GREEN = '\033[38;5;82m'
PURPLE = '\033[38;5;57m'
MAGENTA = '\033[38;5;163m'

# Formatting codes.
RESET = "\033[0;0m"
BOLD = "\033[;1m"
ITALIC = "\033[;3m"
END_ITALIC = "\033[;23m"
REVERSE = "\033[;7m"

# Variable to ensure the welcome message is printed once
# per session.
HAS_PRINTED = False

# ASCII art
illuminati = r'''
      /\      
     /  \     
    /,--.\    
   /< () >\   
  /  `--'  \  
 /          \ 
/------------\
'''



"""Quote repository for portent."""
import random
from datetime import datetime

brass_eye_quotes = {

    # Brass Eye
    "UK used to mean United Kingdom, but ask anyone today and they'll "
    "tell you it stands for Unbelievable Krimewave.": "Chris Morris",

    "And last year, the mayor gave them a goldmine. It actually worked for "
    "a bit, this, until someone clogged it up with sick!": "Ted Maul",

    "What is cake? Well, it has an active ingredient which is a dangerous "
    "psychoactive compound known as "
    "\"dimesmeric andersonphospate\". It stimulates the part of the brain "
    "called \"Shatner's bassoon\", and that's the"
    " bit of the brain that deals with time perception. So a second feels like a month. "
    "Well, it almost sounds like "
    "fun, unless you're the Prague schoolboy who walked out into the street, straight "
    "in front of a tram. "
    "He thought he'd got a month to cross the street...": "Chris Morris",

    "What if a madman broke in here with a machine gun and shot you "
    "to pieces?": "Chris Morris",

    "This is the one thing we *didn't* want to happen.": "Chris Morris",

    "Luckily, the amount of heroin I use is harmless, I inject about once a "
    "month on a purely recreational basis.": "Chris Morris",

    "If you plot 'number of animals abused' against 'what makes people cruel' "
    "versus 'intelligence of either party', "
    "the pattern is so unreadable that you might as well just draw a chain of "
    "fox heads on sticks. And if you do that, "
    "an interesting thing happens: the word 'cruel' starts flashing. So - are "
    "we cruel to hunt foxes?": "Chris Morris",

    "For years, the system of tunnels and shafts has harboured a small "
    "population of wild horses without bothering the commuters.": "Ted Maul",

    "But now, say officials, the horses have become a menace.": "Ted Maul",

    "If time's a drug, then Big Ben is a huge needle injecting it "
    "into the sky.": "Chris Morris",

    "An overdose of heroin is fatal - in the short term. But there has been "
    "no research whatsoever into long term effects.": "Chris Morris",

    "Kids burst shops by filling them with rice, and pouring in water: "
    "then standing back and laughing, while the bricks are ripped apart by "
    "the swelling food.": "Ted Maul",

    "People say that alcohol's a drug. It's not a drug, "
    "it's a drink!": "Chris Morris",

    "Portillo’s teeth removed to boost pound.": "Chris Morris",

    "Nato annulled after delegate swallows treaty.": "Chris Morris",

    "Leicester man wins right to eat sister.": "Chris Morris",

    "Thank goodness, actually, they’re wearing gloves, because I’ve witnessed "
    "bare-knuckle boxing in a barn in Somerset about three years ago. And it "
    "was a sorry sight to see men goading them on in such a barbaric fashion. "
    "And I’m rather ashamed to say I was party to that "
    "goading.": "Alan Partridge",

    "I don’t know if you’ve seen Women In Love. There’s a marvellous scene "
    "by the fire. It kind of resembled that.": "Alan Partridge",

    "The atmosphere here hangs heavy, like a big smell. The smell of men "
    "together.": "Alan Partridge",

    "Due to a printing error, tomorrow’s Telegraph is full of "
    "water.": "Chris Morris",

    "On now to the money markets and a quick look at the International "
    "Finance Arse.": "Collaterlie Sisters",

    "Driving a car with bikes on the roof is not a sportsmanlike way to "
    "compete in the Tour de France.": "Alan Partridge",

    "It's bigger than that Chris, it's large.": "Spartacus Mills",

    "They’ll quite simply say John Major punched the Queen, everything else "
    "will be a footnote.": "Spartacus Mills",

    "You're wrong, and you're a grotesquely ugly freak.": "Chris Morris",

    "From the moon, Cowsick's a little dot. From the ground, it's a huge "
    "mess! Like Dante meets Bosch in a crack lounge!": "Ted Maul",

}


quotes_dict = {
    'satire': brass_eye_quotes,
}


def get_quote_and_attribution_tuple(genre=None):

    if not genre:
        genre = 'satire'

    random.seed(hash(str(datetime.utcnow())) * random.gauss(0, 1))
    d = quotes_dict[genre]
    quote = random.choice(list(d.keys()))
    attribution = d[quote]

    return quote, attribution



from setuptools import setup, find_packages
from os import path
here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='portent',
    version='0.0.1',

    description="Python package providing horrifying "
                "welcome messages for your application",

    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/hypernormalisation/prologue',

    author='Stephen Ogilvy',
    author_email='sogilvy@tuta.io',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],

    keywords='welcome',
    packages=find_packages(exclude=[]),
    python_requires='>=3.6',
    project_urls={
        'Source': 'https://github.com/hypernormalisation/portent',
        'Bug Reports': 'https://github.com/hypernormalisation/timbermafia/portent',
    },
)
